# -*- coding: utf-8 -*-

from django.shortcuts import render, redirect, get_object_or_404
from django.http import Http404
from django.contrib import messages

from lib.tools import confirmation

from ldap_inp import LdapEleve, LdapEleveAvecPhoto
from lib import forms_inp
from lib.access_auth import permission_needed
import datetime

from gestion.eleves.forms import EleveForm, INFOS_CONTACT_ADMIN
from accueil.forms import InfosContactForm
from portailuser.app_logger import AppLogger
from gestion.eleves.models import ChangePhoto
from ldap_inp.ldapobject import L

import io
import base64
from PIL import Image

# Logs
_logger = AppLogger("gestion_eleves")


@permission_needed("gerer_eleves")
def index(request):
    photos = ChangePhoto.objects.filter(user__ecole=request.ecole)

    return render(request, "gestion/eleves/index.html", {"photos": photos})


@permission_needed("gerer_eleves")
def sup_dem_photo(request, ident):
    change = get_object_or_404(ChangePhoto.objects.filter(user__ecole=request.ecole), pk=ident)

    if request.method == "POST":
        if "conf_ok" in request.POST:
            change.delete()
            messages.success(request, "Demande supprimée")
        return redirect("gestion:eleves:index")

    message = "Supprimer cette demande de changement de photo ?"
    boutons = (("conf_annul", "Annuler"), ("conf_ok", "Supprimer"))
    return confirmation(request, message, boutons)


def accepter_liste_demande(liste, request):
    for ident in liste:
        try:
            change = ChangePhoto.objects.filter(user__ecole=request.ecole).get(pk=ident)
        except ChangePhoto.DoesNotExist:
            continue
        eleve = LdapEleveAvecPhoto.objects.get(uid=change.user.username)
        eleve.jpegPhoto = Image.open(io.BytesIO(base64.b64decode(change.new_photo)))
        eleve.save()

        _logger.log(
            request.user,
            "A accepté la nouvelle photo de profil de l'étudiant « {} »",
            change.user.username,
        )
        change.delete()


def delete_liste_demande(liste, request):
    for ident in liste:
        try:
            change = ChangePhoto.objects.filter(user__ecole=request.ecole).get(pk=ident)
        except ChangePhoto.DoesNotExist:
            continue
        change.delete()


@permission_needed("gerer_eleves")
def accept_dem_photo(request, ident):
    get_object_or_404(ChangePhoto.objects.filter(user__ecole=request.ecole), pk=ident)

    if request.method == "POST":
        if "conf_ok" in request.POST:
            accepter_liste_demande([ident], request)
            messages.success(request, "Photo changée")
        return redirect("gestion:eleves:index")

    message = "Accepter cette demande de changement de photo ?"
    boutons = (("conf_annul", "Annuler"), ("conf_ok", "Accepter"))
    return confirmation(request, message, boutons)


@permission_needed("gerer_eleves")
def mod_masse(request):
    print(request.method, request.POST.getlist("mod"))

    if request.method == "POST" and request.POST.getlist("mod"):
        if "Accepter" in request.POST:
            accepter_liste_demande(request.POST.getlist("mod"), request)
            messages.success(request, "Photos changées")

        elif "Rejeter" in request.POST:
            delete_liste_demande(request.POST.getlist("mod"), request)
            messages.success(request, "Demandes supprimées")

    return redirect("gestion:eleves:index")


@permission_needed("gerer_eleves")
def fiche(request, uid):
    try:
        if request.user.check_permission(request, "root"):
            eleve = LdapEleve.objects.get(uid=uid)
        else:
            eleve = LdapEleve.objects.get(uid=uid, ecole=request.ecole)
    except LdapEleve.DoesNotExist:
        raise Http404

    # If a user that is not root try to edit a superuser, redirect to the user' s page
    if eleve.is_superuser() and not request.user.check_permission(request, "root"):
        return redirect("annuaire:fiche", uid=uid)

    if request.method == "POST" and "edit_fiche" in request.POST:
        form = EleveForm(request.POST, request.FILES)
        if form.is_valid():
            donnees = dict(eleve.__dict__)

            for key, val in form.cleaned_data.items():
                if key in form.fields and not isinstance(form.fields[key], forms_inp.ReadOnlyField):
                    if key == "genre":  # ChoiceField retourne une string
                        val = int(val)
                    if key == "birthdate":
                        val = datetime.datetime.combine(val, datetime.datetime.min.time())
                    if key != "chg_photo" and getattr(eleve, key) != val:
                        _logger.log(
                            request.user,
                            "A modifié le champ {} de l'étudiant « {} »",
                            key,
                            uid,
                        )

                    donnees[key] = val

            if "chg_photo" in request.FILES:
                donnees["jpegPhoto"] = form.chg_photo_image
                eleve = LdapEleveAvecPhoto(**donnees)

                _logger.log(
                    request.user,
                    "A modifié la photo de profil de l'étudiant « {} »",
                    uid,
                )
            else:
                eleve = LdapEleve(**donnees)

            eleve.save()

            return redirect("gestion:eleves:fiche", uid=uid)
    else:
        donnees = {
            "uid": eleve.uid,
            "loginTP": eleve.loginTP,
            "nom": eleve.nom,
            "prenom": eleve.prenom,
            "genre": eleve.genre,
            "birthdate": eleve.birthdate,
            "promo": eleve.promo,
            "ecole": eleve.ecole.get().displayName,
            "filiere": eleve.filiere.get().displayName,
            "inscritAE": eleve.inscritAE,
            "inscritFrappe": eleve.inscritFrappe,
            "inscritPassVieEtudiant": eleve.inscritPassVieEtudiant,
            "finPassVieEtudiant": eleve.finPassVieEtudiant,
            "mailEcole": eleve.mailEcole,
            "loginShell": eleve.loginShell,
            "mail": eleve.mail,
        }

        form = EleveForm(donnees)

    # le champs mailLocals ne peux être édité que par un admin INP
    infos_contact = INFOS_CONTACT_ADMIN.copy()
    if not request.user.is_superuser:
        del infos_contact["mailLocal"]

    if request.method == "POST" and "info_contact" in request.POST:
        form_contact = InfosContactForm(eleve, request.POST, infos=infos_contact)

        if form_contact.is_valid() and not form_contact.bouton_presse:
            donnees = form_contact.donnees_entrees()
            for champ in infos_contact.keys():
                try:
                    valeurs = donnees[champ]
                except KeyError:
                    valeurs = []
                if champ == "mailLocal":
                    mails_eleve = getattr(eleve, champ)
                    champ_valide = []
                    for mail in valeurs:
                        # on vérifie qu'il n'y a pas de doublon et que l'adresse n'est pas déjà associé
                        if (mail not in champ_valide) and not LdapEleve.objects.filter(
                            L(mailLocal=mail) & ~L(uid=eleve.uid)
                        ):
                            champ_valide.append(mail)
                            if mail not in mails_eleve:  # rajout d'une adresse
                                _logger.log(
                                    request.user,
                                    "A ajouté l'email {} à « {} »",
                                    mail,
                                    uid,
                                )
                        elif LdapEleve.objects.filter(L(mailLocal=mail) & ~L(uid=eleve.uid)):
                            messages.error(
                                request,
                                f"Impossible de rajouter l'adresse {mail}. Elle est déjà associée à un autre utilisateur",
                            )

                    for mail in mails_eleve:
                        if mail not in champ_valide:  # suppression d'une adresse
                            _logger.log(
                                request.user,
                                "A supprimé l'email {} à « {} »",
                                mail,
                                uid,
                            )
                    setattr(eleve, champ, champ_valide)
                else:
                    if valeurs != getattr(eleve, champ):
                        _logger.log(
                            request.user,
                            "A modifié le champ {} de l'étudiant « {} »",
                            champ,
                            uid,
                        )
                        setattr(eleve, champ, valeurs)

            eleve.save()

            return redirect("gestion:eleves:fiche", uid=uid)
    else:
        form_contact = InfosContactForm(eleve, infos=infos_contact)

    c = {"form": form, "uid": eleve.uid, "form_contact": form_contact}

    return render(request, "gestion/eleves/fiche.html", c)


class Inscrit(object):
    def __init__(self, request, eleve):
        self.prenom = eleve.prenom
        self.nom = eleve.nom
        self.ecole = request.ecole
        filiere = eleve.filiere.get()
        self.aff_filiere = filiere.displayName
        self.promo = eleve.promo
        self.login = eleve.uid
        self.inscrit_ae = eleve.inscritAE
        self.inscrit_pass_vie_etudiant = eleve.inscritPassVieEtudiant
        self.fin_pass_vie_etudiant = eleve.finPassVieEtudiant
        self.est_inscrit = True
        if len(self.aff_filiere) > 20:
            self.shortFiliere = filiere.shortName
        else:
            self.shortFiliere = self.aff_filiere


@permission_needed("gerer_eleves")
def fiche_inscr(request, uid):
    try:
        eleve = LdapEleveAvecPhoto.objects.get(uid=uid, ecole=request.ecole)
    except LdapEleveAvecPhoto.DoesNotExist:
        raise Http404

    inscrit = Inscrit(request, eleve)

    c = {
        "inscr": inscrit,
        "shortFiliere": inscrit.shortFiliere,
        "photo_inscrit": bool(eleve.jpegPhoto),
    }

    return render(request, "gestion/inscriptions/fiche.html", c)
